function [res_struct] = ConfigPendulumModel()
% Configure the pendulum model that serves as ground truth.
%
% --
% Self-tuning LQR, Trimpe et al. IFAC World Congress 2014 paper
%
% Copyright:
% Sebastian Trimpe
% Max-Planck-Institute for Intelligent Systems
% Autonomous Motion Department
% strimpe(at)tuebingen.mpg.de
%
% Revision history
% [13.11.13]   first version to be published


%% Configuration
% Set parameters.

% Model parameters
param_m = 5.3;  % pendulum mass, []=kg
param_l = 0.65; % pendulum length, []=m
param_g = 9.81; % gravity constant, []=m/s^2

Ts = 1/100;     % sampling time

% Noise variances
sigma_n_sq = 0.001;     % Noise on control input, variance
V_n = diag([0 2.5e-6]); % Measurement noise variance
                        % According to the sensor setup used for the
                        % pendulum experiment in Sec. 5, noise on the 
                        % pendulum angle measurements can basically be
                        % neglected (high resolution angle encoders).  The
                        % main source of noise are the rate gyros measuring
                        % the pendulum angular velocity.


%% Linear pendulum model
% Continuous time model (see paper).
A = [0 1; param_g/param_l 0];
B = [0; 1/(param_m*param_l^2)];
sysc = ss(A,B,eye(2),0);

% Discretize at a sampling rate of 1/100s.
sysd = c2d(sysc,Ts);


%% Relevant results
res_struct.sysd = sysd;             % Discrete-time state space model
res_struct.sigma_n_sq = sigma_n_sq; % process noise parameter
res_struct.V_n = V_n;               % measurement noise variance