function [x_opt, J_opt, res_struct] = SPSA(cost_fcn, constr_proj_fcn, x_init, specs)
% Implementation of SPSA optimizer.
%
% Inputs:
%   cost_fcn        cost function handle
%   contr_proj_fcn  constraint projection function; projects parameters 
%                   \theta such that they are in the feasible region \Theta
%   x_init          initial guess for optimizer, row vector
%   specs           SPSA tuning parameters
%
% Outpus:
%   x_opt       optimizer, resulting parameter 
%   J_opt       objective function value at optimizer
%   res_struct  struct storing some results
%
% (Notice that x is used to denote the parameter vector \theta in this
% implementation.)
%
% The SPSA algorithm is according to:
% J. C. Spall. Introduction to Stochastic Search and Optimization: 
% Estimation, Simulation, and Control. Wiley, 2003, Chapter 7.
%
% --
% Self-tuning LQR, Trimpe et al. IFAC World Congress 2014 paper
%
% Copyright:
% Sebastian Trimpe
% Max Planck Institute for Intelligent Systems
% Autonomous Motion Department
% strimpe(at)tuebingen.mpg.de
% 
% Alexander Millane
% Institute for Dynamic Systems and Control
% ETH Zurich
% millanea(at)student.ethz.ch
%
%
% Revision history
% [13.11.13]   first version to be published



%% Optimization

% Initializations
stopOptim = false;      % stop optimization
x_i = x_init;           % current iterate
i = 0;                  % iteration counter
n_costEval = 0;         % number of cost evaluations

dim = length(x_init);   % problem dimension


% Do initial cost evaluation
J_i = cost_fcn(x_i);

% Save results
res_struct(i+1).fval = J_i;
res_struct(i+1).theta_i = x_i;
res_struct(i+1).a_i = nan;
res_struct(i+1).c_i = nan;
res_struct(i+1).report = [];


% Run until stopped
while(~stopOptim)
    % Update optimizer gain variables
    i=i+1;      % next iteration
    
    % Some variables used for reporting optimizer process (can be used for
    % analysis)
    rep = [];
    rep.projection = [];        % indicator if projection has happened (for reping)
    rep.fvals = [];             % for reporting: function evaluations
    rep.fvals_corrected = [];   % for reporting: function evaluations, corrected in case of unstability
    rep.ghats = [];             % individual gradient evaluations
    
    % Gains, fix per iteration
    % Note: in paper, i starts at 0, here at 1.
    a_i = specs.a/(i+specs.a_bar)^specs.alpha;  % Optimization step
    c_i = specs.c/(i)^specs.gamma;          % Gradient approx. step

   
    % Compute gradient approximation.
    
    % Initialize gradient approximation
    ghat = zeros(1,dim);
    
    for iGrad=1:specs.n_g   % Avarage over spec.n_g gradient evaluations

        % Draw random perturbation vector; use Bernoulli RV +/-1, equally
        % likely.
        Delta = 2*round(rand(1,dim))-1;
        
        % Project x_{i+1} if necessary.
        %
        % ... at +c_i*Delta
        x_plus = constr_proj_fcn(x_i+c_i*Delta); 
        % Analysis: x_i was projected due to constraint violation?
        rep.projection = [rep.projection, ~all(x_plus==x_i+c_i*Delta)];
        %
        % ... at -c_i*Delta
        x_minus = constr_proj_fcn(x_i-c_i*Delta);
        % Analysis: x_i was projected due to constraint violation?
        rep.projection = [rep.projection, ~all(x_minus==x_i-c_i*Delta)];
        
        % Function evaluations
        y(iGrad,1) = cost_fcn(x_plus);
        y(iGrad,2) = cost_fcn(x_minus);
        n_costEval = n_costEval+2;  % increment function evaluation counter
        % Analysis:
        rep.fvals = [rep.fvals, y(iGrad,1), y(iGrad,2)];

        
        % Check if any cost is inf (i.e. system is unstable).
        % If yes, use a heuristic:
        %   - cost in one direction is inf -> set it to twice the cost of
        %   the other direction
        %   - costs in both directions are inf -> set both to a large
        %   number
        if isinf(y(iGrad,1))
            if isinf(y(iGrad,2))
                % Both are inf
                y(iGrad,1) = 10;
                y(iGrad,2) = 10;
            else
                % only f(x_plus) is inf; set f(x_plus)=2*f(x_minus)
                y(iGrad,1) = 2*y(iGrad,2);
            end;
        elseif isinf(y(iGrad,2))
            % only f(x_minus) is inf; set f(x_minus)=2*f(x_plus)
            y(iGrad,2) = 2*y(iGrad,1);
        end;
        rep.fvals_corrected = [rep.fvals_corrected, y(iGrad,1), y(iGrad,2)];
        
        
        % Compute approx. gradient
        % 
        % Note that the gradient approximation is inaccurate if x_plus 
        % and/or x_minus have been projected (because then their difference
        % is not 2*c^i*Delta^i).  We neglect this here and always
        % divide by the nominal step size 2*c_i in order to avoid
        % numerical issues if the difference x_plus-x_minus is small.
        ghat_temp = (y(iGrad,1)-y(iGrad,2))/2/c_i*Delta;
        rep.ghats = [rep.ghats; ghat_temp];
        ghat = ghat_temp + ghat;
        
    end;
    ghat = ghat/specs.n_g;     % Gradient approximation
    
    % Update x
    x_i_temp = x_i -a_i*ghat;
    x_i = constr_proj_fcn(x_i_temp);
    rep.projection = [rep.projection, ~all(x_i==x_i_temp)];

    % Function value at new iterate
    % Note: this evaluation might be omitted - it is not necessary to
    % continue the algorithm.
    J_i = cost_fcn(x_i);
    n_costEval = n_costEval+1;
    
    % Save results
    res_struct(i+1).fval = J_i;
    res_struct(i+1).theta_i = x_i;
    res_struct(i+1).a_i = a_i;
    res_struct(i+1).c_i = c_i;
    res_struct(i+1).report = rep;
    
    
    % Termination condition
    if i>=specs.i_max
        stopOptim = true;
    end;
end;

% Results
x_opt = x_i;
J_opt = J_i;
